//TCC_RDC
import { world, system } from "@minecraft/server";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";
import timezones from "./timezones/index.js";
const timezoneNames = [];
if (world.getDynamicProperty("tcc_rdc:timezone") === undefined) world.setDynamicProperty("tcc_rdc:timezone", 191);
if (world.getDynamicProperty("tcc_rdc:settings_item") === undefined) world.setDynamicProperty("tcc_rdc:settings_item", true);
if (world.getDynamicProperty("tcc_rdc:realtime") === undefined) world.setDynamicProperty("tcc_rdc:realtime", true);
if (world.getDynamicProperty("tcc_rdc:dodaylightcycle") === undefined) world.setDynamicProperty("tcc_rdc:dodaylightcycle", true);
if (world.getDynamicProperty("tcc_rdc:initialSetup") === undefined) world.setDynamicProperty("tcc_rdc:initialSetup", true);
function timezoneDisplay(timezone) {
    if (typeof timezone === "string") {
        return world.getDynamicProperty("tcc_rdc:timezone"); //utc format
    } else {
        return timezones[world.getDynamicProperty("tcc_rdc:timezone")].name; //index format
    }
}
for (const timezone of timezones){
    timezoneNames.push(timezone.label);
}
world.afterEvents.itemUse.subscribe(function(event) {
    switch(event.itemStack.typeId){
        case "tcc_rdc:settings":
            if (world.getDynamicProperty("tcc_rdc:settings_item")) selectConfigurationType(event.source);
            break;
    }
});
system.afterEvents.scriptEventReceive.subscribe(function(event) {
    if (event.sourceEntity.typeId != "minecraft:player") return;
    switch(event.id){
        case "tcc_rdc:settings":
            selectConfigurationType(event.sourceEntity);
            break;
        case "tcc_rdc:admin":
            adminConfigurationMenu(event.sourceEntity);
            break;
        case "tcc_rdc:reset":
            world.setDynamicProperty("tcc_rdc:timezone", 191);
            world.setDynamicProperty("tcc_rdc:settings_item", true);
            world.setDynamicProperty("tcc_rdc:realtime", true);
            world.setDynamicProperty("tcc_rdc:dodaylightcycle", true);
            world.setDynamicProperty("tcc_rdc:initialSetup", true);
            break;
    }
});
function selectConfigurationType(player) {
    const ui = new ActionFormData();
    ui.title({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configuration_type.title"
            }
        ]
    }).button({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configuration_type.list"
            }
        ]
    }).button({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configuration_type.manual"
            }
        ]
    }).show(player).then((form)=>{
        if (form.canceled) {
            if (world.getDynamicProperty("tcc_rdc:initialSetup") && world.getDynamicProperty("tcc_rdc:settings_item")) {
                system.runTimeout(()=>{
                    selectConfigurationType(player);
                }, 20);
            }
            //console.log("failed setup")
            return;
        }
        if (form.selection === 0) {
            configurationMenu(player);
        } else {
            configurationMenuManual(player);
        }
    });
}
function configurationMenu(player) {
    const ui = new ModalFormData();
    ui.title({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configure.title"
            }
        ]
    }).dropdown({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configure.currentselection",
                with: [
                    `${timezoneDisplay(world.getDynamicProperty("tcc_rdc:timezone"))}`
                ]
            },
            {
                text: `§r\n\n\n`
            },
            {
                translate: "tcc_rdc.ui.configure.timezone"
            }
        ]
    }, timezoneNames, typeof world.getDynamicProperty("tcc_rdc:timezone") === "number" ? world.getDynamicProperty("tcc_rdc:timezone") : 191).toggle({
        rawtext: [
            {
                translate: "tcc_rdc.ui.admin.realtime"
            }
        ]
    }, world.getDynamicProperty("tcc_rdc:realtime")).show(player).then((form)=>{
        if (form.canceled) {
            if (world.getDynamicProperty("tcc_rdc:initialSetup") && world.getDynamicProperty("tcc_rdc:settings_item")) {
                system.runTimeout(()=>{
                    selectConfigurationType(player);
                }, 20);
            }
            //console.log("failed setup")
            return;
        }
        if (form.formValues[1]) {
            world.sendMessage(`§7${timezones[form.formValues[0]].name}`);
        }
        world.setDynamicProperty("tcc_rdc:timezone", form.formValues[0]);
        world.setDynamicProperty("tcc_rdc:realtime", form.formValues[1]);
        world.setDynamicProperty("tcc_rdc:dodaylightcycle", !form.formValues[1]);
        world.setDynamicProperty("tcc_rdc:initialSetup", false);
    });
}
function configurationMenuManual(player) {
    const ui = new ModalFormData();
    ui.title({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configure.title"
            }
        ]
    }).textField({
        rawtext: [
            {
                translate: "tcc_rdc.ui.configure.currentselection",
                with: [
                    `${timezoneDisplay(world.getDynamicProperty("tcc_rdc:timezone"))}`
                ]
            },
            {
                text: `§r\n\n`
            },
            {
                translate: "tcc_rdc.ui.configure_manual.explaination"
            },
            {
                text: `§r\n\n`
            },
            {
                translate: "tcc_rdc.ui.configure.timezone"
            }
        ]
    }, {
        rawtext: [
            {
                text: `§7§o`
            },
            {
                translate: "tcc_rdc.ui.configure_manual.example"
            }
        ]
    }).toggle({
        rawtext: [
            {
                translate: "tcc_rdc.ui.admin.realtime"
            }
        ]
    }, world.getDynamicProperty("tcc_rdc:realtime")).show(player).then((form)=>{
        if (form.canceled) {
            if (world.getDynamicProperty("tcc_rdc:initialSetup") && world.getDynamicProperty("tcc_rdc:settings_item")) {
                system.runTimeout(()=>{
                    selectConfigurationType(player);
                }, 20);
            }
            //console.log("failed setup")
            return;
        }
        if (isValidTimeOffset(form.formValues[0])) {
            world.sendMessage(`§7${form.formValues[0]}`);
            world.setDynamicProperty("tcc_rdc:timezone", form.formValues[0]);
        } else if (form.formValues[1]) {
            world.sendMessage({
                rawtext: [
                    {
                        translate: "tcc_rdc.ui.configure_manual.invalid",
                        with: [
                            `§7${form.formValues[0]}`
                        ]
                    }
                ]
            });
            system.runTimeout(()=>{
                selectConfigurationType(player);
            }, 20);
        }
        world.setDynamicProperty("tcc_rdc:realtime", form.formValues[1]);
        world.setDynamicProperty("tcc_rdc:dodaylightcycle", !form.formValues[1]);
        world.setDynamicProperty("tcc_rdc:initialSetup", false);
    });
}
function adminConfigurationMenu(player) {
    const ui = new ModalFormData();
    ui.title({
        rawtext: [
            {
                translate: "tcc_rdc.ui.admin.title"
            }
        ]
    }).toggle({
        rawtext: [
            {
                translate: "tcc_rdc.ui.admin.settings_item"
            }
        ]
    }, world.getDynamicProperty("tcc_rdc:settings_item")).toggle({
        rawtext: [
            {
                translate: "tcc_rdc.ui.admin.realtime"
            }
        ]
    }, world.getDynamicProperty("tcc_rdc:realtime")).toggle({
        rawtext: [
            {
                translate: "tcc_rdc.ui.admin.dodaylightcycle"
            }
        ]
    }, world.getDynamicProperty("tcc_rdc:dodaylightcycle")).show(player).then((form)=>{
        if (form.canceled) return;
        world.sendMessage({
            rawtext: [
                {
                    translate: "tcc_rdc.ui.admin.settings_saved"
                }
            ]
        });
        world.setDynamicProperty("tcc_rdc:settings_item", form.formValues[0]);
        world.setDynamicProperty("tcc_rdc:realtime", form.formValues[1]);
        world.setDynamicProperty("tcc_rdc:dodaylightcycle", form.formValues[2]);
    });
}
function getCurrentTimeWithOffset(offsetStr) {
    const regex = /^([+-])(\d{2}):(\d{2})$/;
    const match = offsetStr.match(regex);
    if (!match) return;
    const sign = match[1] === '+' ? 1 : -1;
    const hours = parseInt(match[2], 10);
    const minutes = parseInt(match[3], 10);
    const offsetMilliseconds = sign * ((hours * 60 + minutes) * 60 * 1000);
    const nowUtcMilliseconds = Date.now();
    const targetTimestamp = nowUtcMilliseconds + offsetMilliseconds;
    return new Date(targetTimestamp);
}
function getDayProgress(offset) {
    const now = getCurrentTimeWithOffset(offset);
    const startOfDay = new Date(now.getFullYear(), now.getMonth(), now.getDate());
    const msSinceMidnight = now.getTime() - startOfDay.getTime();
    const msInDay = 24 * 60 * 60 * 1000;
    const fractionOfDay = msSinceMidnight / msInDay;
    return (fractionOfDay * 24000 + 18000) % 24000;
}
world.afterEvents.playerSpawn.subscribe((event)=>{
    if (event.initialSpawn && world.getDynamicProperty("tcc_rdc:initialSetup") && world.getDynamicProperty("tcc_rdc:settings_item")) {
        //console.log(event.player.name, "timezone setup")
        selectConfigurationType(event.player);
    }
});
system.runInterval(()=>{
    if (world.getDynamicProperty("tcc_rdc:realtime")) {
        world.gameRules.doDayLightCycle = false;
        if (typeof world.getDynamicProperty("tcc_rdc:timezone") === "string") {
            world.setTimeOfDay(getDayProgress(world.getDynamicProperty("tcc_rdc:timezone"))) //utc format
            ;
        } else {
            world.setTimeOfDay(getDayProgress(timezones[world.getDynamicProperty("tcc_rdc:timezone")].utc)) //index format
            ;
        }
    } else if (world.getDynamicProperty("tcc_rdc:dodaylightcycle")) {
        world.gameRules.doDayLightCycle = true;
    } else {
        world.gameRules.doDayLightCycle = false;
    }
}, 1);
function isValidTimeOffset(offset) {
    const regex = /^[+-](0[0-9]|1[0-2]):([0-5][0-9])$/;
    if (!regex.test(offset)) {
        return false;
    }
    const sign = offset[0];
    const [hours, minutes] = offset.slice(1).split(":").map(Number);
    return minutes >= 0 && minutes <= 59;
}
